@@ -43,7 +43,7 @@ use strict;
 ########### USER CONFIGURABLE SECTION #######################
 
 my $newdir = "xml";
-my $rrdtool = "/scratch/oetiker/rrd/bin/rrdtool";
+my $rrdtool = "/usr/local/rrdtool-1.0.50/bin/rrdtool";
 my $add_ds = "./add_ds.pl";  # path to add_ds.pl script
 
 ########### END CONFIGURE SECTION ###########################
