@@ -12,7 +12,7 @@
 #ifndef _PHP4_RRDTOOL_H
 #define _PHP4_RRDTOOL_H
 
-#if COMPILE_DL
+#if COMPILE_DL_RRDTOOL
 #undef HAVE_RRDTOOL
 #define HAVE_RRDTOOL 1
 #endif
