@@ -6,7 +6,7 @@ use strict;
 use lib qw( ../../perl-shared/blib/lib ../../perl-shared/blib/arch );
 
 #makes programm work AFTER install
-use lib qw( /scratch/oetiker/rrd/lib/perl ../lib/perl );
+use lib qw( /usr/local/rrdtool-1.0.50/lib/perl ../lib/perl );
 
 use vars qw(@ISA $loaded);
 
