@@ -1,5 +1,5 @@
 /*****************************************************************************
- * RRDtool 1.0.49  Copyright Tobias Oetiker, 1997 - 2004
+ * RRDtool 1.0.50  Copyright Tobias Oetiker, 1997 - 2004
  *****************************************************************************
  * rrd_tool.c  Startup wrapper
  *****************************************************************************/
@@ -20,12 +20,12 @@ void PrintUsage(char *cmd)
 {
 
     char help_main[] =
-	   "RRDtool 1.0.49  Copyright 1997-2004 by Tobias Oetiker <tobi@oetiker.ch>\n\n"
+	   "RRDtool 1.0.50  Copyright 1997-2004 by Tobias Oetiker <tobi@oetiker.ch>\n\n"
 	   "Usage: rrdtool [options] command command_options\n\n";
 
     char help_list[] =
 	   "Valid commands: create, update, graph, dump, restore,\n"
-	   "\t\tlast, info, fetch, tune, resize, xport\n\n";
+	   "\t\tlast, first, info, fetch, tune, resize, xport\n\n";
 
     char help_create[] =
 	   "* create - create a new RRD\n\n"
@@ -49,6 +49,10 @@ void PrintUsage(char *cmd)
            "* last - show last update time for RRD\n\n"
            "\trrdtool last filename.rrd\n\n";
 
+    char help_first[] =
+           "* first - show first update time for RRA within an RRD\n\n"
+           "\trrdtool first filename.rrd [--rraindex number]\n\n";
+
     char help_update[] =
 	   "* update - update an RRD\n\n"
 	   "\trrdtool update filename\n"
@@ -120,7 +124,7 @@ void PrintUsage(char *cmd)
 
 	   "For more information read the RRD manpages\n\n";
 
-    enum { C_NONE, C_CREATE, C_DUMP, C_INFO, C_RESTORE, C_LAST,
+    enum { C_NONE, C_CREATE, C_DUMP, C_INFO, C_RESTORE, C_LAST, C_FIRST,
 	   C_UPDATE, C_FETCH, C_GRAPH, C_TUNE, C_RESIZE, C_XPORT };
 
     int help_cmd = C_NONE;
@@ -137,6 +141,8 @@ void PrintUsage(char *cmd)
 		help_cmd = C_RESTORE;
     	    else if (!strcmp(cmd,"last"))
 		help_cmd = C_LAST;
+    	    else if (!strcmp(cmd,"first"))
+		help_cmd = C_FIRST;
     	    else if (!strcmp(cmd,"update"))
 		help_cmd = C_UPDATE;
     	    else if (!strcmp(cmd,"fetch"))
@@ -171,6 +177,9 @@ void PrintUsage(char *cmd)
 	    case C_LAST:
 		fputs(help_last, stdout);
 		break;
+	    case C_FIRST:
+		fputs(help_first, stdout);
+		break;
 	    case C_UPDATE:
 		fputs(help_update, stdout);
 		break;
@@ -263,8 +272,11 @@ int main(int argc, char *argv[])
 		PrintUsage(argv[1]);
 		exit(0);
 	}
-    else
-	HandleInputLine(argc, argv, stderr);    
+     else {
+        if(HandleInputLine(argc, argv, stderr)) {
+            return 1;
+        }
+    }
     return 0;
 }
 
@@ -322,13 +334,15 @@ int HandleInputLine(int argc, char **argv, FILE* out)
 	     strcmp("v", argv[1]) == 0 ||
 	     strcmp("-v", argv[1]) == 0  ||
 	     strcmp("-version", argv[1]) == 0  )
-        printf("RRDtool 1.0.49  Copyright (C) 1997-2004 by Tobias Oetiker <tobi@oetiker.ch>\n");
+        printf("RRDtool 1.0.50  Copyright (C) 1997-2004 by Tobias Oetiker <tobi@oetiker.ch>\n");
     else if (strcmp("restore", argv[1]) == 0)
 	rrd_restore(argc-1, &argv[1]);
     else if (strcmp("resize", argv[1]) == 0)
 	rrd_resize(argc-1, &argv[1]);
     else if (strcmp("last", argv[1]) == 0)
         printf("%ld\n",rrd_last(argc-1, &argv[1]));
+    else if (strcmp("first", argv[1]) == 0)
+        printf("%ld\n",rrd_first(argc-1, &argv[1]));
     else if (strcmp("update", argv[1]) == 0)
 	rrd_update(argc-1, &argv[1]);
     else if (strcmp("fetch", argv[1]) == 0) {
@@ -355,8 +369,8 @@ int HandleInputLine(int argc, char **argv, FILE* out)
 	}
     } else if (strcmp("xport", argv[1]) == 0) {
 	int xxsize;
-	int i = 0, j = 0;
-	time_t        start,end;
+	int j;
+	time_t        i,start,end;
 	unsigned long step, col_cnt,row_cnt;
 	rrd_value_t   *data,*ptr;
 	char          **legend_v;
@@ -426,6 +440,7 @@ int HandleInputLine(int argc, char **argv, FILE* out)
     if (rrd_test_error()) {
 	fprintf(out, "ERROR: %s\n",rrd_get_error());
 	rrd_clear_error();
+	return 1;
     }
     return(0);
 }
